# react-gui

Hooks are the mechanism to work with events, gestures, and other behaviors. This allows for easier composition of behaviors (vs using components and merging props) and helps to limit churn in the component APIs. These Hooks can support the implementation of the [WAI-ARIA Authoring Practices](https://www.w3.org/TR/wai-aria-practices-1.2/) in a reusable way.

## Components

The foundational components are adapted from those used in React Native today. These components can adopt behaviors by linking refs to the event and interaction hooks.

### View

View is the fundamental layout primitive. It has built in accessibility controls, enforces flexbox columns by default (and supports grid and float layout). Raw text nodes are not allowed as children or siblings of View, and no text-related styling is supported. A View nested within a Text will render inline without altering its display or that of its children. The full set of accessibility props can be set on a `View` (e.g., `accessibilityRole`, etc.). In addition, it has the following props:

* `direction`: set the direction of the layout to `ltr` or `rtl`.
* `focusable`: make the view focusable with a keyboard.
* `href`: make the view render as a link.
* `hrefAttrs`: configure the link behavior.
* `nativeID`: set the native `id`.
* `style`: apply strict styles (not typographic styles).
* `testID`: label the element with a test selector.

### Text

Text is the fundamental text primitive. It has built in accessibility controls, enforces inline layout by default, provides limited inheritance of text styles, and supports View as children. It inherits typographic styles from ancestor `Text` elements (as long as the chain of text elements is not interrupted by a `View`). By default, text is rendered using the native platform’s algorithm to determine the directionality of the content language. The full set of accessibility props can be set on a `Text` (e.g., `accessibilityRole`, etc.). In addition, it has the following props:

* `direction`: set the writing direction of the text to `ltr` or `rtl`.
* `focusable`: make the text focusable with a keyboard.
* `href`: make the text render as an inline link.
* `hrefAttrs`: configure the link behavior.
* `lang`: the language of the content text.
* `nativeID`: set the native `id`.
* `style`: apply strict styles (including typographic styles).
* `testID`: label the element with a test selector.

## APIs

### StyleSheet

Work with strict styles that provide deterministic rendering and automatically adapt to localized writing direction.

```js
import View from 'react-gui/view';
import StyleSheet from 'react-gui/style-sheet';

function Component() {
  return <View style={styles.example} />
}

const styles = StyleSheet.create({
  example: {
    borderWidth: 5
  }
});
```

## Interactions

High-level interactions that simplify working with accessible behaviors across devices and platforms. These hooks (and the underlying event hooks) make it easy to work with pointer gestures, keyboard interactions, and layout changes to adjust a component's render tree and styles. The interaction hooks cause a component to update when their return value changes (if this is not desired, use the event hooks).

### useActiveModality

Reflects the modality that is currently active: `keyboard`, `mouse`, `touch`, `pen`.

```js
import View from 'react-gui/view';
import useActiveModality from 'react-gui/use-active-modality';

function Component() {
  const { activeModality } = useActiveModality();

  return (
    <View
      ref={ref}
      style={[
        isFocused && styles.focus
      ]}
    />
  );
}
```


### useFocus

Reflects whether the element is focused.

```js
import View from 'react-gui/view';
import useFocus from 'react-gui/use-focus';

function Component() {
  const ref = React.useRef(null);
  const { isFocused } = useFocus(ref);

  return (
    <View
      ref={ref}
      style={[
        isFocused && styles.focus
      ]}
    />
  );
}
```

### useFocusVisible

Reflects whether the element is visibly focused. `isFocusVisible` is true when the focus visibility of the target element changes, i.e., it is called when the element is focused with a keyboard, and when the element either loses focus or is interacted with using a pointer. `isFocusVisibleWithin` is also true when these changes occur in the *subtree* of the element.

```js
import View from 'react-gui/view';
import useFocusVisible from 'react-gui/use-focus-visible';

function Component() {
  const ref = React.useRef(null);
  const { isFocusVisible, isFocusVisibleWithin } = useFocusVisible(ref);

  return (
    <View
      ref={ref}
      style={[
        isFocusVisible && styles.focusRing,
        // example: only change styles if the element itself is not visibly focused
        !isFocusVisible && isFocusVisibleWithin && styles.focusWithinRing
      ]}
    />
  );
}
```

### useFocusWithin

Reflects whether the element is focused or contains the currently focused element.

```js
import View from 'react-gui/view';
import useFocusWithin from 'react-gui/use-focus-within';

function Component() {
  const ref = React.useRef(null);
  const { isFocusWithin } = useFocusWithin(ref);

  return (
    <View
      ref={ref}
      style={[
        isFocusWithin && styles.focusWithinRing
      ]}
    />
  );
}
```

### useHover

Reflects whether the element is hovered and (optionally) where the pointer is on screen.

```js
import View from 'react-gui/view';
import useHover from 'react-gui/use-hover';

function Component() {
  const ref = React.useRef(null);
  const { isHovered, x, y } = useHover(ref, {
    contain: true,
    updates: true
  });

  return (
    <View
      ref={ref}
      style={[
        width > 100 && { flexDirection: 'row' }
      ]}
    />
  );
}
```

### useLayout

Reflects the `border-box` layout of the element.

```js
import View from 'react-gui/view';
import useLayout from 'react-gui/use-layout';

function Component() {
  const ref = React.useRef(null);
  const { x, y, width, height } = useLayout(ref);

  return (
    <View
      ref={ref}
      style={[
        width > 100 && { flexDirection: 'row' }
      ]}
    />
  );
}
```

## Events

Working with events from within components is done with hooks. The framework provides platform-agnostic high-level event abstractions for common gestures, as well as providing web-specific, low-level APIs for working directly with specific native events.

### useClickEvents(ref, config)

Listen to "click" events on the element. The `e.detail` property reflects how many clicks have occurred in short succession.

```js
import View from 'react-gui/view';
import useClickEvents from 'react-gui/use-click-events';

function Component() {
  const ref = React.useRef(null);

  useClickEvents(ref, {
    onClick,
    onClickCapture
  });

  return <View ref={ref} />
}
```

### useFocusEvents(ref, config)

Listen to "focus" events on the element. The `blur` and `focus` events do not bubble.

```js
import View from 'react-gui/view';
import useFocusEvents from 'react-gui/use-focus-events';

function Component() {
  const ref = React.useRef(null);

  useFocusEvents(ref, {
    onBlur,
    onBlurCapture,
    onFocus,
    onFocusCapture,
    onFocusChange
  });

  return <View ref={ref} />
}
```

### useFocusWithinEvents(ref, config)

Listen to "focus" events that occur within the subtree of the element. `onBlurWithin` and `onFocusWithin` are called *each time* the focus moves within the subtree, whereas `onFocusWithinChange` is only called once when focus enters and leaves the entire subtree.

```js
import View from 'react-gui/view';
import useFocusWithinEvents from 'react-gui/use-focus-within-events';

function Component() {
  const ref = React.useRef(null);

  useFocusWithinEvents(ref, {
    onBlurWithin,
    onFocusWithin,
    onFocusWithinChange
  });

  return <View ref={ref} />
}
```

### useHoverEvents(ref, config)

Listen to pointer "hover" events on the element. These events are only fired for modalities that can hover over elements, e.g., 'mouse' interactions. Hover is not accessible to touch or keyboard interactions. Hover events bubble and the `contain` prop should be used to contain hover events to a specific subtree.

```js
import View from 'react-gui/view';
import useHoverEvents from 'react-gui/use-hover-events';

function Component() {
  const ref = React.useRef(null);

  useHoverEvents(ref, {
    contain: true,
    onHoverChange,
    onHoverStart,
    onHoverUpdate,
    onHoverEnd,
  });

  return <View ref={ref} />
}
```

### useKeyboardEvents(ref, config)

Listen to "keyboard" events on the element. These events are only fired when IME composition is not taking place.

```js
import View from 'react-gui/view';
import useKeyboardEvents from 'react-gui/use-keyboard-events';

function Component() {
  const ref = React.useRef(null);

  useKeyboardEvents(ref, {
    onKeyDown,
    onKeyDownCapture,
    onKeyUp,
    onKeyUpCapture
  });

  return <View ref={ref} />
}
```

### useLayoutEvents(ref, config)

Listen to changes in "layout" on the element. The `onLayout` event will return a layout object corresponding to the `border-box` dimensions, and `onLayoutContent` the `context-box` dimensions. (This hook is implemented with `ResizeObserver`.)

```js
import View from 'react-gui/view';
import useLayoutEvents from 'react-gui/use-layout-events';

function Component({ onLayout, onLayoutContent }) {
  const ref = React.useRef(null);

  useLayoutEvents(ref, {
    onLayout,
    onLayoutContent
  });

  return <View ref={ref} />
}
```

<!--
### usePanEvents(ref, config)

Listen to pointer "pan" events on the element. These events are only fired for pointers, i.e., these interactions are not accessible to keyboards. (This hook is implemented with `useResponderEvents`.)

```js
import View from 'react-gui/view';
import usePanEvents from 'react-gui/use-pan-events';

function Component() {
  const ref = React.useRef(null);

  usePanEvents(ref, {
    onStartShouldSet,
    onMoveShouldSet(e, gestureState) {
      // start gesture once centroid of pointers has moved horizontally
      if (MathgestureState.deltaX > 10) {
        return true;
      }
    },
    onPanStart,
    onPanUpdate,
    onPanEnd,
    onPanCancel
  });

  return <View ref={ref} />
}
```
-->
<!--
### usePressEvents(ref, config)

Listen to "press" events on the element. The press event collection combines "tap", "keyboard", and "click" interactions, providing an accessible way to listen to user interactions from all modality types. The `onPress` event is equivalent to the native `click` event, which means it may be called in isolation (by assistive technology, etc.) and it may not be bookended by `onPressStart` and `onPressEnd`.

```js
import View from 'react-gui/view';
import usePressEvents from 'react-gui/use-tap-events';

function Component() {
  const ref = React.useRef(null);

  usePressEvents(ref, {
    onPressChange,
    onPressStart,
    onPressUpdate,
    onPressEnd,
    onPressCancel,
    onPress
  });

  return <View acessibilityRole="button" ref={ref} />
}
```
-->

### useResponderEvents(ref, config)

Listen to pointer "responder" events on the element. See the [Responder System README](./use-responder-events/README.nd) for documentation.

```js
import View from 'react-gui/view';
import useResponderEvents from 'react-gui/use-responder-events';

function Component() {
  const ref = React.useRef(null);

  useResponderEvents(ref, {
    disabled,
    onStartShouldSetResponder,
    onResponderGrant,
    onResponderStart,
    onResponderMove,
    onResponderEnd,
    onResponderRelease,
    onResponderTerminate,
  });

  return <View ref={ref} />
}
```

<!--
### useTapEvents(ref, config)

Listen to pointer "tap" events on the element. These events are only fired for pointers, i.e., these interactions are not accessible to keyboards. For generic "press" (aka "click") interactions you should use `usePressEvents`. (This hook is implemented with `useResponderEvents`.)

```js
import View from 'react-gui/view';
import useTapEvents from 'react-gui/use-tap-events';

function Component() {
  const ref = React.useRef(null);

  useTapEvents(ref, {
    onTapChange,
    onTapStart,
    onTapUpdate,
    onTapEnd,
    onTapCancel
  });

  return <View ref={ref} />
}
```
-->

## Imperative methods

Methods for working with refs to host elements.

### activeModality()

Returns the modality last used, e.g., `keyboard`, `mouse`, `touch`, `pen`.

```js
import { activeModality } from 'react-gui';

const activeModality = activeModality();
```

### blur(ref)

Blur an element.

```js
import { blur } from 'react-gui';
import View from 'react-gui/view';

function Component() {
  const ref = React.useRef(null);

  function doSomething() {
    if (ref != null && blurCondition) {
      blur(ref);
    }
  });

  return <View ref={ref} />
}
```

### clear(textInputRef)

Clear the text from a text input.

```js
import { clear } from 'react-gui';
import TextInput from 'react-gui/text-input';

function Component() {
  const ref = React.useRef(null);

  function doSomething() {
    if (ref != null && clearCondition) {
      clear(ref);
    }
  });

  return <TextInput ref={ref} />
}
```

### focus(ref, options)

Focus an element.

```js
import { focus } from 'react-gui';
import View from 'react-gui/view';

function Component() {
  const ref = React.useRef(null);

  function doSomething() {
    if (ref != null && focusCondition) {
      focus(ref, { preventScroll: true });
    }
  });

  return <View ref={ref} />
}
```

### getLayout(ref, includeTransform)

Measure the layout (border-box dimensions) of an element. By default the dimension do not account for transforms. Pass `true` as the second argument to include CSS layout transforms in the result.

```js
import { getLayout } from 'react-gui';
import View from 'react-gui/view';

function Component() {
  const ref = React.useRef(null);

  function doSomething() {
    if (ref != null && getLayoutcondition) {
      const { x, y, width, height } = getLayout(ref);
    }
  });

  return <View ref={ref} />
}
```

## Unstable DOM APIs

These unstable APIs are the DOM-specific abstractions used to create the built-in, platform-agnostic APIs.

### createElement(type, props)

Create a fundamental element with the core accessibility and style props. The module provides low-level tools that can be used to create custom "fundamental" components on the web. The core accessibility and styling props can be filtered by `filterProps`, and "unsafe" CSS resets can be created with `createStyleRule`.

```js
import createElement, {createStyleRules, filterProps} from 'react-gui/create-element';

function View(props) {
  // Only the props recognized by 'createElement'
  const filteredProps = createProps(props);
  // Attach a CSS reset
  filteredProps.classList = [ css.base ]
  // Add specific props to forward to the DOM
  filteredProps.dir = props.direction;

  return createElement('div', filteredProps);
}

const css = createStyleRules({
  base: {
    display: 'flex',
    ...
  }
});
```

### createEventHandle(type, options)

Listen to arbitrary DOM events from outside of React components. This should be rarely needed. It is available to provide direct access to the web platform's event system for building high-level systems like the built-in "Responder System" (for negotiating ownership over pointers) and the "Modality System" (for tracking changes in how a person is interacting with the UI).

```js
import createEventHandle from 'react-gui/create-event-handle';

const clickHandle = createEventHandle('click', options);
clickHandle.setListener(node, clickListener);
clickHandle.setListener(anotherNode, anotherClickListener);
clickHandle.clear()
```

### useEvent(type, options)

Listen to arbitrary DOM events on an element. This is the hook-equivalent of `createEventHandle` and would generally only be used to create custom event hooks to complement the built-in event hooks. Note that the first argument to the `setListener` method on the returned event handle is a *host node*, i.e., the value of the ref and not the ref itself.

```js
import useEvent from 'react-gui/use-event';

function useClick(ref, { onClick }) {
  const clickHandle = useEvent('click');
  React.useLayoutEffect(() => {
    const node = ref.current;
    if (node != null) {
      // Listeners must be set in effects or event handlers
      clickHandle.setListener(node, onClick);
    }
  }, ref, onClick);
}
```

## Appendix

### Accessibility props

Accessibility props map directly to WAI-ARIA properties (excluding properties and values that are deprecated, redundant, or to be avoided).

* `accessibilityActiveDescendant`
* `accessibilityAtomic`
* `accessibilityAutoComplete`
* `accessibilityBusy`
* `accessibilityChecked`
* `accessibilityColumnCount`
* `accessibilityColumnIndex`
* `accessibilityColumnSpan`
* `accessibilityControls`
* `accessibilityDescribedBy`
* `accessibilityDetails`
* `accessibilityDisabled`
* `accessibilityErrorMessage`
* `accessibilityExpanded`
* `accessibilityFlowTo`
* `accessibilityHasPopup`
* `accessibilityHidden`
* `accessibilityInvalid`
* `accessibilityKeyShortcuts`
* `accessibilityLabel`
* `accessibilityLabelledBy`
* `accessibilityLevel`
* `accessibilityLiveRegion`
* `accessibilityModal`
* `accessibilityMultiline`
* `accessibilityMultiSelectable`
* `accessibilityOrientation`
* `accessibilityOwns`
* `accessibilityPlaceholder`
* `accessibilityPosInSet`
* `accessibilityPressed`
* `accessibilityReadOnly`
* `accessibilityRequired`
* `accessibilityRole`
* `accessibilityRoleDescription`
* `accessibilityRowCount`
* `accessibilityRowIndex`
* `accessibilityRowSpan`
* `accessibilitySelected`
* `accessibilitySetSize`
* `accessibilitySort`
* `accessibilityValueMax`
* `accessibilityValueMin`
* `accessibilityValueNow`
* `accessibilityValueText`
