/**
 * Copyright (c) Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
'use strict';

exports.__esModule = true;
exports.default = createEventHandle;

var _supportsPassiveEvents = _interopRequireDefault(require("../__modules__/supportsPassiveEvents"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var canUsePassiveEvents = (0, _supportsPassiveEvents.default)();

function getOptions(options) {
  if (options == null) {
    return false;
  }

  return canUsePassiveEvents ? options : Boolean(options.capture);
}

function addListener(target, type, listener, opts) {
  target.addEventListener(type, listener, opts);
  return function removeListener() {
    if (target != null) {
      target.removeEventListener(type, listener, opts);
    }
  };
}
/**
 * Shim generic API compatibility with ReactDOM's synthetic events, without needing the
 * large amount of code ReactDOM uses to do this. Ideally we wouldn't use a synthetic
 * event wrapper at all.
 */


function normalizeEvent(event) {
  var defaultPrevented = false;
  var propagationStopped = false;
  var nativePreventDefault = event.preventDefault;
  var nativeStopPropagation = event.stopPropagation;
  event.nativeEvent = event;

  event.persist = function () {};

  function preventDefault() {
    nativePreventDefault.call(event);
    defaultPrevented = true;
    Object.defineProperty(event, 'defaultPrevented', {
      value: true
    });
  }

  Object.defineProperty(event, 'preventDefault', {
    configurable: true,
    value: preventDefault
  });

  function stopPropagation() {
    nativeStopPropagation.call(event);
    propagationStopped = true;
  }

  Object.defineProperty(event, 'stopPropagation', {
    configurable: true,
    value: stopPropagation
  });

  event.isDefaultPrevented = function () {
    return defaultPrevented;
  };

  event.isPropagationStopped = function () {
    return propagationStopped;
  };

  return event;
}
/**
 *
 */


function createEventHandle(type, options) {
  var listenerMap = new Map();
  var opts = getOptions(options);
  return {
    setListener: function setListener(target, listener) {
      if (target == null || typeof target.addEventListener !== 'function') {
        throw new Error('createEventHandle: setListener called on an invalid target.');
      }

      var removePreviousListener = listenerMap.get(target);

      if (removePreviousListener != null) {
        removePreviousListener();
      }

      if (listener != null) {
        var compatListener = function compatListener(e) {
          listener(normalizeEvent(e));
        };

        var removeListener = addListener(target, type, compatListener, opts);
        listenerMap.set(target, removeListener);
      }
    },
    clear: function clear() {
      listenerMap.forEach(function (removeListener) {
        removeListener();
      });
    }
  };
}

module.exports = exports.default;