"use strict";

exports.__esModule = true;
exports.attachListeners = attachListeners;
exports.isFocusVisible = isFocusVisible;
exports.addListener = addListener;

var _createEventHandle = _interopRequireDefault(require("../create-event-handle"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright (c) Nicolas Gallagher.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * 
 */
var isGlobalFocusVisible = true;
var changeHandlers = new Set();
var hasSetupGlobalListeners = false;
var hasEventBeforeFocus = false;
var isMac = typeof window !== 'undefined' && window.navigator != null ? /^Mac/.test(window.navigator.platform) : false; // Only Tab or Esc keys will make focus visible on text input elements

var FOCUS_VISIBLE_INPUT_KEYS = {
  Tab: true,
  Escape: true
};

function triggerChangeHandlers(e, info) {
  for (var _iterator = changeHandlers, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var handler = _ref;
    handler(e, info);
  }
} // Helper function to determine if a KeyboardEvent is unmodified and could make keyboard focus styles visible


function isValidKey(e) {
  return !(e.metaKey || !isMac && e.altKey || e.ctrlKey);
}

function handleKeyboardEvent(e) {
  hasEventBeforeFocus = true;

  if (isValidKey(e)) {
    isGlobalFocusVisible = true;
    triggerChangeHandlers(e, {
      modality: 'keyboard',
      isFocusVisible: isGlobalFocusVisible
    });
  }
}

function handlePointerEvent(e) {
  isGlobalFocusVisible = false;

  if (e.type === 'mousedown' || e.type === 'pointerdown' || e.type === 'touchstart') {
    hasEventBeforeFocus = true;
    triggerChangeHandlers(e, {
      modality: 'pointer',
      isFocusVisible: isGlobalFocusVisible
    });
  }
}

function handleFocusEvent(e) {
  // Firefox fires two extra focus events when the user first clicks into an iframe:
  // first on the window, then on the document. We ignore these events so they don't
  // cause keyboard focus rings to appear.
  if (e.target === window || e.target === document) {
    return;
  } // If a focus event occurs without a preceding keyboard or pointer event, switch to keyboard modality.
  // This occurs, for example, when navigating a form with the next/previous buttons on iOS.


  if (!hasEventBeforeFocus) {
    isGlobalFocusVisible = true;
    triggerChangeHandlers(e, {
      modality: 'keyboard',
      isFocusVisible: isGlobalFocusVisible
    });
  }

  hasEventBeforeFocus = false;
}

function handleWindowBlur() {
  // When the window is blurred, reset state. This is necessary when tabbing out of the window,
  // for example, since a subsequent focus event won't be fired.
  hasEventBeforeFocus = false;
} // Setup global event listeners to control when keyboard focus style should be visible


function attachListeners() {
  if (hasSetupGlobalListeners) {
    return;
  } // Programmatic focus() calls shouldn't affect the current input modality.
  // However, we need to detect other cases when a focus event occurs without
  // a preceding user event (e.g. screen reader focus). Overriding the focus
  // method on HTMLElement.prototype is a bit hacky, but works.


  var focus = HTMLElement.prototype.focus;

  HTMLElement.prototype.focus = function () {
    hasEventBeforeFocus = true;
    focus.apply(this, arguments);
  };

  document.addEventListener('keydown', handleKeyboardEvent, true);
  document.addEventListener('keyup', handleKeyboardEvent, true); // Register focus events on the window so they are sure to happen
  // before React's event listeners (registered on the document).

  window.addEventListener('focus', handleFocusEvent, true);
  window.addEventListener('blur', handleWindowBlur, false);

  if (typeof PointerEvent !== 'undefined') {
    document.addEventListener('pointerdown', handlePointerEvent, true);
    document.addEventListener('pointermove', handlePointerEvent, true);
    document.addEventListener('pointerup', handlePointerEvent, true);
  } else {
    document.addEventListener('touchstart', handlePointerEvent, true);
    document.addEventListener('touchmove', handlePointerEvent, true);
    document.addEventListener('touchend', handlePointerEvent, true);
    document.addEventListener('mousedown', handlePointerEvent, true);
    document.addEventListener('mousemove', handlePointerEvent, true);
    document.addEventListener('mouseup', handlePointerEvent, true);
  }

  hasSetupGlobalListeners = true;
}

function isFocusVisible() {
  return isGlobalFocusVisible;
}

function addListener(listener) {
  changeHandlers.add(listener);
  return function () {
    changeHandlers.delete(listener);
  };
}